/****************************************************************************
** Execute class
**
** Created: Tue Jan 26 12:09:08 2003
**      by: Varol Okan using kate editor
**
** This class provide a GUI to the man pages.
**
****************************************************************************/

#ifndef DIALOGMAN_H
#define DIALOGMAN_H

#include <qprocess.h>

#include <qvariant.h>
#include <qvaluelist.h>
#include <qdialog.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QPushButton;
class QStringList;
class QTabWidget;
class QTextEdit;
class QWidget;
class QLineEdit;

class DialogMan : public QDialog
{
    Q_OBJECT

public:
     DialogMan( QStringList &commands, QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~DialogMan();

protected:


protected slots:
	virtual void slotFind ();
	virtual void languageChange();

protected: 
    QPushButton *buttonHelp;
    QPushButton *buttonFind;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;
    QLineEdit   *m_pEditFind;
    QTabWidget  *tabWidget2;
    
    QGridLayout* MyDialogLayout;
    QHBoxLayout* Layout1;
    QGridLayout* tabLayout;
    QGridLayout* tabLayout_2;
	QValueList <QTextEdit *> m_listTextEdit;
};

#endif // DIALOGMAN_H
